@@ -1,15 +1,8 @@
 """
-Bisprectrum calculation using numba acceleration
+Bispectrum CPU implementation.
 
-This implementation works on 2D and 3D rectangular domains for real
-or complex valued data. It can compute the bispectrum exactly or
-using uniform sampling.
+.. moduleauthor:: Michael O'Brien <michaelobrien@g.harvard.edu>
 
-Author:
-    Michael J. O'Brien (2021)
-    Biophysical Modeling Group
-    Center for Computational Biology
-    Flatiron Institute
 """
 
 import numpy as np
@@ -17,66 +10,73 @@ import numba as nb
 from time import time
 
 
-def bispectrum(data, kmin=None, kmax=None, nsamples=None, sample_thresh=None,
-               exclude=False, mean_subtract=False, compute_fft=True,
-               use_pyfftw=False, bench=False, progress=False, **kwargs):
+def bispectrum(data, kmin=None, kmax=None,
+               nsamples=None, sample_thresh=None,
+               exclude=False, mean_subtract=False,
+               compute_fft=True, full=False,
+               use_pyfftw=False,
+               bench=False, progress=False, **kwargs):
     """
     Compute the bispectrum of 2D or 3D real or complex valued data.
 
-    Arguments
-    ---------
+    kwargs are passed to np.fft.fftn, np.fft.rfftn,
+    pyfftw.builders.fftn, or pyfftw.builders.rfftn.
+
+    Parameters
+    ----------
     data : np.ndarray
         Real or complex valued 2D or 3D scalar data.
         The shape should be (d1, d2) or (d1, d2, d3)
         where di is the ith dimension of the image.
-
-    Keywords
-    --------
-    kmin : int
+    kmin : int, optional
         Minimum wavenumber in bispectrum calculation.
-    kmax : int
+    kmax : int, optional
         Maximum wavenumber in bispectrum calculation.
-    nsamples : int, float, or np.ndarray
-        Number of sample triangles to take. This may be
-        an array of shape [kmax-kmin+1, kmax-kmin+1] to
-        specify either 1) the number of samples to take
-        for a given point or 2) the fraction of total
-        possible triangles to sample. If None, calculate
+    nsamples : int, float or np.ndarray, shape (kmax-kmin+1, kmax-kmin+1), optional
+        Number of sample triangles or fraction of total
+        possible triangles. This may be an array that
+        specifies for a given k1, k2. If None, calculate
         the bispectrum exactly.
-    sample_thresh : int
+    sample_thresh : int, optional
         When the size of the sample space is greater than
         this number, start to use sampling instead of exact
         calculation. If None, switch to exact calculation
         when nsamples is less than the size of the sample space.
-    exclude : bool
+    exclude : bool, optional
         If True, exclude k1, k2 such that k1 + k2 is greater
         than the Nyquist frequency. Excluded points will be
         set to nan.
-    mean_subtract : bool
+    mean_subtract : bool, optional
         Subtract mean off of image data to highlight
         non-linearities in bicoherence.
-    compute_fft : bool
+    compute_fft : bool, optional
         If False, do not take the FFT of the input data.
-    use_pyfftw : bool
-        If True, use pyfftw (see function fftn below)
-        to compute the FFTs.
-    bench : bool
+    full : bool, optional
+        Return the full output of calculation. Namely,
+        return the optional sampling diagnostics.
+    use_pyfftw : bool, optional
+        If True, use pyfftw to compute the FFTs.
+    bench : bool, optional
         If True, print calculation time.
-
-    **kwargs passed to fftn (defined below)
+    progress : bool, optional
+        Print progress bar of calculation.
 
     Returns
     -------
-    bispectrum : np.ndarray
-        Complex-valued 2D image
-    bicoherence : np.ndarray
-        Real-valued normalized bispectrum
+    bispec : np.ndarray, shape (kmax-kmin+1, kmax-kmin+1)
+        Real or complex-valued bispectrum.
+        Will be real-valued if the input data is real.
+    bicoh : np.ndarray, shape (kmax-kmin+1, kmax-kmin+1)
+        Real-valued bicoherence.
     kn : np.ndarray
-        Wavenumbers along axis of bispectrum
+        Wavenumbers along axis of bispectrum.
+    omega : np.ndarray, shape (kmax-kmin+1, kmax-kmin+1), optional
+        Number of possible triangles in the sample space.
+    counts : np.ndarray, shape (kmax-kmin+1, kmax-kmin+1), optional
+        Number of evaluations in the bispectrum sum.
     """
-    
+
     shape, ndim = nb.typed.List(data.shape), data.ndim
-    norm = float(data.size)**3
 
     if ndim not in [2, 3]:
         raise ValueError("Data must be 2D or 3D.")
@@ -122,7 +122,7 @@ def bispectrum(data, kmin=None, kmax=None, nsamples=None, sample_thresh=None,
     if compute_fft:
         temp = data - data.mean() if mean_subtract else data
         if use_pyfftw:
-            fft = fftn(temp, **kwargs)
+            fft = _fftn(temp, **kwargs)
         else:
             fft = np.fft.fftn(temp, **kwargs)
         del temp
@@ -144,23 +144,27 @@ def bispectrum(data, kmin=None, kmax=None, nsamples=None, sample_thresh=None,
             nsamples = nsamples.astype(np.int_)
 
     # Run main loop
-    compute_point = compute_point3D if ndim == 3 else compute_point2D
-    bispec, binorm = compute_bispectrum(kind, kn, kcoords, fft,
-                                        nsamples, sample_thresh,
-                                        ndim, dim, shape, progress,
-                                        exclude, compute_point)
+    compute_point = _compute_point3D if ndim == 3 else _compute_point2D
+    args = (kind, kn, kcoords, fft, nsamples, sample_thresh,
+            ndim, dim, shape, progress, exclude, compute_point)
+    bispec, binorm, omega, counts = _compute_bispectrum(*args)
+
+    if np.issubdtype(data.dtype, np.floating):
+        bispec = bispec.real
 
     bicoh = np.abs(bispec) / binorm
-    bispec /= norm
+    bispec *= (omega / counts)
 
     if bench:
         print(f"Time: {time() - t0:.04f} s")
 
-    return bispec, bicoh, kn
+    if not full:
+        return bispec, bicoh, kn
+    else:
+        return bispec, bicoh, kn, omega, counts
 
 
-def fftn(image, overwrite_input=False, threads=-1,
-         dtype=np.complex128, **kwargs):
+def _fftn(image, overwrite_input=False, threads=-1, **kwargs):
     """
     Calculate N-dimensional fft of image with pyfftw.
     See pyfftw.builders.fftn for kwargs documentation.
@@ -169,14 +173,11 @@ def fftn(image, overwrite_input=False, threads=-1,
     ----------
     image : np.ndarray
         Real or complex valued 2D or 3D image
-
-    Keywords
-    --------
-    overwrite_input : bool
+    overwrite_input : bool, optional
         Specify whether input data can be destroyed.
         This is useful for reducing memory usage.
         See pyfftw.builders.fftn for more.
-    threads : int
+    threads : int, optional
         Number of threads for pyfftw to use. Default
         is number of cores.
 
@@ -190,23 +191,33 @@ def fftn(image, overwrite_input=False, threads=-1,
     """
     import pyfftw
 
+    if image.dtype in [np.complex64, np.complex128]:
+        dtype = 'complex128'
+        fftn = pyfftw.builders.fftn
+    elif image.dtype in [np.float32, np.float64]:
+        dtype = 'float64'
+        fftn = pyfftw.builders.rfftn
+    else:
+        raise ValueError(f"{data.dtype} is unrecognized data type.")
+
     a = pyfftw.empty_aligned(image.shape, dtype=dtype)
-    f = pyfftw.builders.fftn(a, overwrite_input=overwrite_input,
-                             threads=threads, **kwargs)
+    f = fftn(a, threads=threads, overwrite_input=overwrite_input, **kwargs)
     a[...] = image
     fft = f()
 
-    del a
+    del a, fftn
 
     return fft
 
 
 @nb.njit(parallel=True)
-def compute_bispectrum(kind, kn, kcoords, fft, nsamples, sample_thresh,
-                       ndim, dim, shape, progress, exclude, compute_point):
+def _compute_bispectrum(kind, kn, kcoords, fft, nsamples, sample_thresh,
+                        ndim, dim, shape, progress, exclude, compute_point):
     knyq = max(shape) // 2
     bispec = np.full((dim, dim), np.nan, dtype=np.complex128)
     binorm = np.full((dim, dim), np.nan, dtype=np.float64)
+    omega = np.zeros((dim, dim), dtype=np.int64)
+    counts = np.zeros((dim, dim), dtype=np.int64)
     for i in range(dim):
         k1 = kn[i]
         k1ind = kind[i]
@@ -233,19 +244,21 @@ def compute_bispectrum(kind, kn, kcoords, fft, nsamples, sample_thresh,
                           nk1, nk2, shape, samp, count,
                           bispecbuf, binormbuf, countbuf)
             N = countbuf.sum()
-            value = nk1*nk2*(bispecbuf.sum() / N)
-            norm = nk1*nk2*(binormbuf.sum() / N)
+            value = bispecbuf.sum()
+            norm = binormbuf.sum()
             bispec[i, j], bispec[j, i] = value, value
             binorm[i, j], binorm[j, i] = norm, norm
+            omega[i, j], omega[j, i] = nk1*nk2, nk1*nk2
+            counts[i, j], counts[j, i] = N, N
         if progress:
             with nb.objmode():
-                printProgressBar(i, dim-1)
-    return bispec, binorm
+                _printProgressBar(i, dim-1)
+    return bispec, binorm, omega, counts
 
 
 @nb.njit(parallel=True, cache=True)
-def compute_point3D(k1ind, k2ind, kcoords, fft, nk1, nk2, shape,
-                    samp, count, bispecbuf, binormbuf, countbuf):
+def _compute_point3D(k1ind, k2ind, kcoords, fft, nk1, nk2, shape,
+                     samp, count, bispecbuf, binormbuf, countbuf):
     kx, ky, kz = kcoords[0], kcoords[1], kcoords[2]
     Nx, Ny, Nz = shape[0], shape[1], shape[2]
     for idx in nb.prange(count):
@@ -262,8 +275,8 @@ def compute_point3D(k1ind, k2ind, kcoords, fft, nk1, nk2, shape,
 
 
 @nb.njit(parallel=True, cache=True)
-def compute_point2D(k1ind, k2ind, kcoords, fft, nk1, nk2, shape,
-                    samp, count, bispecbuf, binormbuf, countbuf):
+def _compute_point2D(k1ind, k2ind, kcoords, fft, nk1, nk2, shape,
+                     samp, count, bispecbuf, binormbuf, countbuf):
     kx, ky = kcoords[0], kcoords[1]
     Nx, Ny = shape[0], shape[1]
     for idx in nb.prange(count):
@@ -280,8 +293,8 @@ def compute_point2D(k1ind, k2ind, kcoords, fft, nk1, nk2, shape,
 
 
 @nb.jit(forceobj=True, cache=True)
-def printProgressBar(iteration, total, prefix='', suffix='', decimals=1,
-                     length=50, fill='█', printEnd="\r"):
+def _printProgressBar(iteration, total, prefix='', suffix='', decimals=1,
+                      length=50, fill='█', printEnd="\r"):
     """
     Call in a loop to create terminal progress bar
 
@@ -303,12 +316,15 @@ if __name__ == '__main__':
     from mpl_toolkits.axes_grid1 import make_axes_locatable
 
     N = 512
+    np.random.seed(1234)
     data = np.random.normal(size=N**2).reshape((N, N))+1
 
-    kmin, kmax = 1, 4
-    bispec, bicoh, kn = bispectrum(data, nsamples=int(5e7),
-                                   kmin=kmin, kmax=kmax, progress=True,
-                                   mean_subtract=True, bench=True, exclude=True)
+    kmin, kmax = 1, 100
+    bispec, bicoh, kn, omega, counts = bispectrum(data, nsamples=None,
+                                                  kmin=kmin, kmax=kmax,
+                                                  progress=True,
+                                                  mean_subtract=True,
+                                                  full=True, bench=True)
     print(bispec.mean(), bicoh.mean())
     print(bicoh.max())
 
@@ -327,8 +343,8 @@ if __name__ == '__main__':
         cax = divider.append_axes("right", size="5%", pad=0.05)
         cbar = plt.colorbar(im, cax=cax)
         cbar.set_label(labels[i])
-        if i == 0:
-            ax.contour(data[i], colors='k', extent=[kmin, kmax, kmin, kmax])
+        #if i == 0:
+        #    ax.contour(data[i], colors='k', extent=[kmin, kmax, kmin, kmax])
         ax.set_xlabel(r"$k_1$")
         ax.set_ylabel(r"$k_2$")
 
