@@ -1,15 +1,8 @@
 """
-Bisprectrum calculation using CUDA acceleration
+Bispectrum GPU implementation.
 
-This implementation works on 2D and 3D rectangular domains for real
-or complex valued data. It can compute the bispectrum exactly or
-using uniform sampling.
+.. moduleauthor:: Michael O'Brien <michaelobrien@g.harvard.edu>
 
-Author:
-    Michael J. O'Brien (2021)
-    Biophysical Modeling Group
-    Center for Computational Biology
-    Flatiron Institute
 """
 
 import numpy as np
@@ -18,13 +11,20 @@ from cupyx.scipy import fft as cufft
 from time import time
 
 
-def bispectrum(data, kmin=None, kmax=None, nsamples=None, sample_thresh=None,
-               double=True, exclude=False, mean_subtract=False, blocksize=128,
-               compute_fft=True, bench=False, progress=False, **kwargs):
+def bispectrum(data, kmin=None, kmax=None,
+               nsamples=None, sample_thresh=None,
+               exclude=False, mean_subtract=False,
+               compute_fft=True, full=False,
+               double=True, blocksize=128,
+               bench=False, progress=False, **kwargs):
     """
-    Compute the bispectrum of 2D or 3D real or complex valued data.
+    Compute the bispectrum of 2D or 3D real or complex valued data
+    using CuPy acceleration using CuPy acceleration.
 
-    Arguments
+    kwargs are passed to cupyx.scipy.fft.fftn
+    or cupyx.scipy.fft.rfftn.
+
+    Parameters
     ---------
     data : np.ndarray
         Real or complex valued 2D or 3D scalar data.
@@ -32,49 +32,55 @@ def bispectrum(data, kmin=None, kmax=None, nsamples=None, sample_thresh=None,
         where di is the ith dimension of the image.
         Can be CPU or GPU data. If it is GPU data and
         complex, it will be overwritten by default.
-
-    Keywords
-    --------
-    kmin : int
+    kmin : int, optional
         Minimum wavenumber in bispectrum calculation.
-    kmax : int
+    kmax : int, optional
         Maximum wavenumber in bispectrum calculation.
-    nsamples : int, float, or np.ndarray
-        Number of sample triangles to take. This may be
-        an array of shape [kmax-kmin+1, kmax-kmin+1] to
-        specify either 1) the number of samples to take
-        for a given point or 2) the fraction of total
-        possible triangles to sample. If None, calculate
+    nsamples : int, float or np.ndarray, shape (kmax-kmin+1, kmax-kmin+1), optional
+        Number of sample triangles or fraction of total
+        possible triangles. This may be an array that
+        specifies for a given k1, k2. If None, calculate
         the bispectrum exactly.
-    sample_thresh : int
+    sample_thresh : int, optional
         When the size of the sample space is greater than
         this number, start to use sampling instead of exact
         calculation. If None, switch to exact calculation
         when nsamples is less than the size of the sample space.
-    double : bool
-        If False, do calculation in single precision.
-    exclude : bool
+    exclude : bool, optional
         If True, exclude k1, k2 such that k1 + k2 is greater
         than the Nyquist frequency. Excluded points will be
         set to nan.
-    mean_subtract : bool
+    mean_subtract : bool, optional
         Subtract mean off of image data to highlight
         non-linearities in bicoherence.
-    compute_fft : bool
+    compute_fft : bool, optional
         If False, do not take the FFT of the input data.
-    bench : bool
+    full : bool, optional
+        If True, return the full output of calculation. Namely,
+        return the optional sampling diagnostics.
+    double : bool, optional
+        If False, do calculation in single precision.
+    blocksize : int, optional
+        Number of threads per block for GPU kernels.
+        The optimal value will vary depending on hardware.
+    progress : bool, optional
+        Print progress bar of calculation.
+    bench : bool, optional
         If True, print calculation time.
 
-    **kwargs passed to cufftn (defined below)
-
     Returns
     -------
-    bispectrum : np.ndarray
-        Complex-valued 2D image
-    bicoherence : np.ndarray
-        Real-valued normalized bispectrum
+    bispec : np.ndarray, shape (kmax-kmin+1, kmax-kmin+1)
+        Real or complex-valued bispectrum.
+        Will be real-valued if the input data is real.
+    bicoh : np.ndarray, shape (kmax-kmin+1, kmax-kmin+1)
+        Real-valued bicoherence.
     kn : np.ndarray
-        Wavenumbers along axis of bispectrum
+        Wavenumbers along axis of bispectrum.
+    omega : np.ndarray, shape (kmax-kmin+1, kmax-kmin+1), optional
+        Number of possible triangles in the sample space.
+    counts : np.ndarray, shape (kmax-kmin+1, kmax-kmin+1), optional
+        Number of evaluations in the bispectrum sum.
     """
 
     if double:
@@ -86,7 +92,6 @@ def bispectrum(data, kmin=None, kmax=None, nsamples=None, sample_thresh=None,
     pinned_mempool = cp.get_default_pinned_memory_pool()
 
     shape, ndim = data.shape, data.ndim
-    norm = float(data.size)**3
 
     if ndim not in [2, 3]:
         raise ValueError("Data must be 2D or 3D.")
@@ -107,8 +112,8 @@ def bispectrum(data, kmin=None, kmax=None, nsamples=None, sample_thresh=None,
     tpb = blocksize
     bpg = (kr.size + (tpb - 1)) // tpb
     for i in range(ndim):
-        sqr_add((bpg,), (tpb,), (kr, kv[i], kr.size))
-    sqrt((bpg,), (tpb,), (kr, kr.size))
+        _sqr_add((bpg,), (tpb,), (kr, kv[i], kr.size))
+    _sqrt((bpg,), (tpb,), (kr, kr.size))
 
     # Convert coordinates to int16
     kcoords = []
@@ -149,7 +154,7 @@ def bispectrum(data, kmin=None, kmax=None, nsamples=None, sample_thresh=None,
         temp = cp.asarray(data, dtype=complex)
         if mean_subtract:
             temp[...] -= temp.mean()
-        fft = cufftn(temp, **kwargs)
+        fft = _cufftn(temp, **kwargs)
         del temp
     else:
         fft = data.astype(complex, copy=False)
@@ -183,22 +188,27 @@ def bispectrum(data, kmin=None, kmax=None, nsamples=None, sample_thresh=None,
 
     # Run main loop
     f = "" if double else "f"
-    compute_point = module.get_function(f"compute_point{ndim}D{f}")
-    bispec, binorm = compute_bispectrum(kind, kn, kcoords, fft,
-                                        nsamples, sample_thresh,
-                                        ndim, dim, shape, double, progress,
-                                        exclude, blocksize, compute_point)
+    compute_point = _module.get_function(f"compute_point{ndim}D{f}")
+    args = (kind, kn, kcoords, fft, nsamples, sample_thresh, ndim,
+            dim, shape, double, progress, exclude, blocksize, compute_point)
+    bispec, binorm, omega, counts = _compute_bispectrum(*args)
 
-    bicoh = cp.abs(bispec) / binorm
-    bispec /= norm
+    if np.issubdtype(data.dtype, np.floating):
+        bispec = bispec.real
+
+    bicoh = np.abs(bispec) / binorm
+    bispec *= (omega / counts)
 
     if bench:
         print(f"Time: {time() - t0:.04f} s")
 
-    return bispec.get(), bicoh.get(), kn
+    if not full:
+        return bispec, bicoh, kn
+    else:
+        return bispec, bicoh, kn, omega, counts
 
 
-def cufftn(data, overwrite_input=True, **kwargs):
+def _cufftn(data, overwrite_input=True, **kwargs):
     """
     Calculate the N-dimensional fft of an image
     with memory efficiency
@@ -207,15 +217,13 @@ def cufftn(data, overwrite_input=True, **kwargs):
     ----------
     data : cupy.ndarray
         Real or complex valued 2D or 3D image.
-
-    Keywords
-    --------
-    overwrite_input : bool
+    overwrite_input : bool, optional
         Specify whether input data can be destroyed.
         This is useful if low on memory.
         See cupyx.scipy.fft.fftn for more.
 
-    **kwargs passes to cupyx.scipy.fft.fftn or cupyx.scipy.fft.rfftn
+    **kwargs passes to cupyx.scipy.fft.fftn or
+    cupyx.scipy.fft.rfftn
 
     Returns
     -------
@@ -250,7 +258,7 @@ def cufftn(data, overwrite_input=True, **kwargs):
     return fft
 
 
-sqr_add = cp.RawKernel(r'''
+_sqr_add = cp.RawKernel(r'''
 #include <cupy/carray.cuh>
 
 extern "C" __global__
@@ -270,7 +278,7 @@ void square_add(float* kr, float* ki, int size) {
 ''', 'square_add')
 
 
-sqrt = cp.RawKernel(r'''
+_sqrt = cp.RawKernel(r'''
 #include <cupy/carray.cuh>
 
 extern "C" __global__
@@ -290,9 +298,9 @@ void square_root(float* kr, int size) {
 ''', 'square_root')
 
 
-def compute_bispectrum(kind, kn, kcoords, fft, nsamples, sample_thresh,
-                       ndim, dim, shape, double, progress,
-                       exclude, blocksize, compute_point):
+def _compute_bispectrum(kind, kn, kcoords, fft, nsamples, sample_thresh,
+                        ndim, dim, shape, double, progress,
+                        exclude, blocksize, compute_point):
     knyq = max(shape) // 2
     shape = [cp.int16(Ni) for Ni in shape]
     if double:
@@ -303,6 +311,8 @@ def compute_bispectrum(kind, kn, kcoords, fft, nsamples, sample_thresh,
     pinned_mempool = cp.get_default_pinned_memory_pool()
     bispec = cp.full((dim, dim), cp.nan+1.j*cp.nan, dtype=complex)
     binorm = cp.full((dim, dim), cp.nan, dtype=float)
+    omega = np.zeros((dim, dim), dtype=np.int64)
+    counts = cp.zeros((dim, dim), dtype=cp.int64)
     for i in range(dim):
         k1 = kn[i]
         k1ind = kind[i]
@@ -333,20 +343,22 @@ def compute_bispectrum(kind, kn, kcoords, fft, nsamples, sample_thresh,
                                            *shape, samp, cp.int64(count),
                                            bispecbuf, binormbuf, countbuf))
             N = countbuf.sum()
-            value = nk1*nk2*(bispecbuf.sum() / N)
-            norm = nk1*nk2*(binormbuf.sum() / N)
+            value = bispecbuf.sum()
+            norm = binormbuf.sum()
             bispec[i, j], bispec[j, i] = value, value
             binorm[i, j], binorm[j, i] = norm, norm
+            omega[i, j], omega[j, i] = nk1*nk2, nk1*nk2
+            counts[i, j], counts[j, i] = N, N
             del bispecbuf, binormbuf, countbuf, samp
             mempool.free_all_blocks()
             pinned_mempool.free_all_blocks()
         if progress:
-            printProgressBar(i, dim-1)
+            _printProgressBar(i, dim-1)
 
-    return bispec, binorm
+    return bispec.get(), binorm.get(), omega, counts.get()
 
 
-module = cp.RawModule(code=r'''
+_module = cp.RawModule(code=r'''
 # include <cupy/complex.cuh>
 
 extern "C" {
@@ -584,8 +596,8 @@ __global__ void compute_point2Df(const long* k1ind, const long* k2ind,
 }''')
 
 
-def printProgressBar(iteration, total, prefix='', suffix='', decimals=1,
-                     length=50, fill='█', printEnd="\r"):
+def _printProgressBar(iteration, total, prefix='', suffix='', decimals=1,
+                      length=50, fill='█', printEnd="\r"):
     """
     Call in a loop to create terminal progress bar
 
@@ -607,12 +619,15 @@ if __name__ == '__main__':
     from mpl_toolkits.axes_grid1 import make_axes_locatable
 
     N = 512
+    np.random.seed(1234)
     data = np.random.normal(size=N**2).reshape((N, N))+1
 
-    kmin, kmax = 1, 32
-    bispec, bicoh, kn = bispectrum(data, nsamples=int(5e7),
-                                   kmin=kmin, kmax=kmax, progress=True,
-                                   mean_subtract=True, bench=True, exclude=True)
+    kmin, kmax = 1, 100
+    bispec, bicoh, kn, omega, counts = bispectrum(data, nsamples=None,
+                                                  kmin=kmin, kmax=kmax,
+                                                  progress=True, double=True,
+                                                  mean_subtract=True,
+                                                  full=True, bench=True)
     print(bispec.mean(), bicoh.mean())
     print(bicoh.max())
 
